/* FGGE - FabioGiopla Game Engine
 * Coded by Fabio Iotti
 */

function FGGE()
{
	var root = this;

	this.clickMouse = false;
	
	this.mouseX = 0;
	this.mouseY = 0;
	
	this.keyDown = new Array();
	
	this.Canvas;
	this.Context2D;
	
	this.FPS = 15;
	
	this.totalExternals = 0;
	this.loadedExternals = 0;
	
	var startingGame = false;
	
	var gameCycleIntervalID;
	var gameCycleFunction;
	
	var readyFunction;
	
	this.Layer = new Array();
	
	function getCursorPosition(e) {
		// SNIPPET: http://tinyurl.com/3r6kcqf
		var x;
		var y;
		if (e.pageX || e.pageY) {
			x = e.pageX;
			y = e.pageY;
		}
		else {
			x = e.clientX + document.body.scrollLeft +
					document.documentElement.scrollLeft;
			y = e.clientY + document.body.scrollTop +
					document.documentElement.scrollTop;
		}

		x -= root.Canvas.offsetLeft;
		y -= root.Canvas.offsetTop;

		return [x,y];
	}
	
	function upDateMouse(e) {
		root.mouseX = getCursorPosition(e)[0];
		root.mouseY = getCursorPosition(e)[1];
	}
	
	function upDateMouseDown(e) {
		root.clickMouse = true;
	}
	
	function upDateMouseUp(e) {
		root.clickMouse = false;
	}
	
	function upDateKeyDown(e) {
		root.keyDown[e.keyCode] = true;
	}
	
	function upDateKeyUp(e) {
		root.keyDown[e.keyCode] = false;
	}
	
	this.makeEnvironment = function(tag) {
		root.Canvas = document.createElement("canvas");
		tag.appendChild(root.Canvas);

		root.Context2D = root.Canvas.getContext('2d');
	};
	
	this.setBackgroundColor = function(color) {
		root.Canvas.style.backgroundColor = color;
	};
	
	this.addLayer = function(layer) {
		root.Layer.push(layer);
	};
	
	this.loadImage = function(image) {
		root.totalExternals++;
		image.onload = function() {
			root.loadedExternals++;
			
			if(root.loadedExternals >= root.totalExternals) {
				if(startingGame) {
					root.forceStartGame();
				}
			}
		};
		
		image.onerror = function() {
			alert("WARNING: Failed to load \""+image.src+"\" image!");
			
			root.loadedExternals++;
			
			if(root.loadedExternals >= root.totalExternals) {
				if(startingGame) {
					root.forceStartGame();
				}
			}
		};
	};
	
	this.callWhenReady = function(rdfun) {
		readyFunction = rdfun;
	};
	
	this.startGame = function() {
		startingGame = true;
		
		if(root.loadedExternals >= root.totalExternals) {			
			root.forceStartGame();
		}
	};
	
	this.forceStartGame = function() {
		if(readyFunction)
			readyFunction();
	
		startingGame = false;
		
		if(!gameCycleIntervalID)
			gameCycleIntervalID = setInterval(root.updateGame, 1000/root.FPS);
			
		root.Canvas.addEventListener("mousemove", upDateMouse, false);
		root.Canvas.addEventListener("mouseup", upDateMouseUp, false);
		root.Canvas.addEventListener("mousedown", upDateMouseDown, false);
		
		document.addEventListener("keydown", upDateKeyDown, false);
		document.addEventListener("keyup", upDateKeyUp, false);
	};
	
	this.addKeyListener = function(keyListenerFunction) {
		document.addEventListener("keydown", keyListenerFunction, false);
	};
	
	this.updateGame = function() {
		if(gameCycleFunction) {
			try {
				gameCycleFunction();
			}
			catch(error) {
				clearInterval(gameCycleIntervalID);
				gameCycleIntervalID = undefined;
				
				alert(error);
			}
		}
		
		root.clearScreen();
		
		try {
			for(var layerNum = 0; layerNum < root.Layer.length; layerNum++) {
				for(var modelNum = 0; modelNum < root.Layer[layerNum].Model.length; modelNum++) {
					if(!root.Layer[layerNum].Model[modelNum].hidden) {
						if(root.Layer[layerNum].Model[modelNum].rot) {
							root.Context2D.save();
							
							root.Context2D.translate(
							root.Layer[layerNum].Model[modelNum].x,
							root.Layer[layerNum].Model[modelNum].y
							);
							
							root.Context2D.rotate(root.Layer[layerNum].Model[modelNum].rot);
							
							root.Context2D.drawImage(
							root.Layer[layerNum].Model[modelNum].getImage(),
							0,
							0,
							root.Layer[layerNum].Model[modelNum].getWidth(),
							root.Layer[layerNum].Model[modelNum].getHeight(),
							-root.Layer[layerNum].Model[modelNum].centerX,
							-root.Layer[layerNum].Model[modelNum].centerY,
							root.Layer[layerNum].Model[modelNum].getWidth(),
							root.Layer[layerNum].Model[modelNum].getHeight()
							);
							
							root.Context2D.restore();
						}
						else {
							root.Context2D.drawImage(
							root.Layer[layerNum].Model[modelNum].getImage(),
							root.Layer[layerNum].Model[modelNum].x - root.Layer[layerNum].Model[modelNum].centerX,
							root.Layer[layerNum].Model[modelNum].y - root.Layer[layerNum].Model[modelNum].centerY
							);
						}
						
						// TODO: Implement scale.
					}
				}
			}
		}
		catch(error) {
			alert("Layer: "+layerNum+"\nModel: "+modelNum+"\n\n"+error);
		}
	};
	
	this.setGameCycleFunction = function(gcFunction) {
		gameCycleFunction = gcFunction;
	}
	
	this.clearScreen = function() {
		root.Context2D.clearRect(0, 0, root.Canvas.width, root.Canvas.height);
	}
}

function Layer()
{
	this.Model = new Array();
	
	this.clearLayer = function() {
		this.Model = new Array();
	}
	
	this.addModel = function(model) {
		this.Model.push(model);
	};
}

function Model()
{
	var root = this;
	
	this.x = 0;
	this.y = 0;
	
	this.xScale;
	this.yScale;
	
	this.centerX = 0;
	this.centerY = 0;
	
	this.alpha;
	
	this.hidden;
	
	this.rot;
	
	var image;
	
	this.detectPointCollision = function(pointX, pointY) {
		if(pointX > root.x - root.centerX && pointX < root.x - root.centerX + this.getWidth()) {
			if(pointY > root.y - root.centerY && pointY < root.y - root.centerY + this.getHeight()) {
				return true;
			}
		}
		
		return false;
	};
	
	this.getImage = function() {
		return image;
	};
	
	this.getWidth = function() {
		return image.width;
	};
	
	this.getHeight = function() {
		return image.height;
	};
	
	this.applyImage = function(imageo) {
		image = imageo;
	};
}
